'use strict';

var _Object$assign = require('babel-runtime/core-js/object/assign')['default'];

exports.__esModule = true;
exports['default'] = flatten;

var _isArray$isObject$isString = require('./utils');

var notEmpty = function notEmpty(x) {
  return x !== '' && x != null;
};
var mergeArrays = function mergeArrays(a, b) {
  return (a.push.apply(a, b), a);
};
var mergeObjects = function mergeObjects(a, b) {
  return _Object$assign(a, b);
};
var concatenate = function concatenate(a, b) {
  return a + b;
};

var merge = function merge(a, b) {
  if (a.constructor === b.constructor) {
    if (_isArray$isObject$isString.isArray(a)) return mergeArrays(a, b);
    if (_isArray$isObject$isString.isObject(a)) return mergeObjects(a, b);
    return a + b;
  }

  if (_isArray$isObject$isString.isString(a)) return b;
  if (_isArray$isObject$isString.isString(b)) return a;
  if (_isArray$isObject$isString.isArray(a)) return (a.push(b), a);
  if (_isArray$isObject$isString.isArray(b)) return (b.push(a), b);

  throw new Error('Unexpected case');
};

function flattenSequence(values) {
  return values.filter(notEmpty).reduce(merge, '');
}

function flattenRepetition(values, named) {
  if (named && values.length === 0) {
    return [];
  }

  if (values.some(_isArray$isObject$isString.isObject)) {
    return values.filter(_isArray$isObject$isString.isObject);
  }

  if (values.some(_isArray$isObject$isString.isArray)) {
    return values.filter(_isArray$isObject$isString.isArray).reduce(mergeArrays, []);
  }

  return values.reduce(concatenate, '');
}

function flatten(values) {
  var named = arguments[1] === undefined ? false : arguments[1];

  if (!_isArray$isObject$isString.isArray(values)) {
    return values;
  }

  var tag = values[0];
  var tail = values.slice(1);

  var result = tail.map(function (x) {
    return flatten(x);
  });

  switch (tag) {
    case 'maybe':
      return named ? result[0] : result[0] || '';
    case 'repetition':
      return flattenRepetition(result, named);
    case 'sequence':
      return flattenSequence(result);
  }

  throw new Error('Unknown tag \'' + tag + '\'');
}

module.exports = exports['default'];