'use strict';

var _classCallCheck = require('babel-runtime/helpers/class-call-check')['default'];

var _Object$assign = require('babel-runtime/core-js/object/assign')['default'];

exports.__esModule = true;

// Match API
// ---------

exports.subtree = subtree;
exports.simple = simple;
exports.sequence = sequence;

var _isArray$isObject$mapValues = require('./utils');

function subtree(node) {
  return node != null;
}

function simple(node) {
  return subtree(node) && !_isArray$isObject$mapValues.isArray(node) && !_isArray$isObject$mapValues.isObject(node);
}

function sequence(node) {
  return subtree(node) && _isArray$isObject$mapValues.isArray(node) && node.every(simple);
}

// Transform API
// -------------

var Transform = (function () {
  function Transform() {
    var rules = arguments[0] === undefined ? [] : arguments[0];

    _classCallCheck(this, Transform);

    this.rules = rules;
  }

  Transform.prototype.rule = function rule(match, transform) {
    this.rules.push([match, transform]);
    return this;
  };

  Transform.prototype.run = function run(node, context) {
    var _this = this;

    var runInContext = function runInContext(node) {
      return _this.run(node, context);
    };

    if (_isArray$isObject$mapValues.isArray(node)) {
      node = node.map(runInContext);
    } else if (_isArray$isObject$mapValues.isObject(node)) {
      node = _isArray$isObject$mapValues.mapValues(node, runInContext);
    }

    for (var i = 0; i < this.rules.length; i++) {
      var _rules$i = this.rules[i];
      var match = _rules$i[0];
      var transform = _rules$i[1];

      if (match(node, context)) {
        return transform(node, context);
      }
    }

    return node;
  };

  return Transform;
})();

exports['default'] = Transform;

var Node = function Node(data) {
  _classCallCheck(this, Node);

  if (_isArray$isObject$mapValues.isObject(data)) {
    _Object$assign(this, data);
  }
};

exports.Node = Node;

// require() compatibility
// -----------------------

Transform.Node = Node;
Transform.subtree = subtree;
Transform.simple = simple;
Transform.sequence = sequence;